C$Procedure      FILBUF (Fill buffer)

      SUBROUTINE FILBUF (UNIT, START, NLIN, BUFFER, EOF)

C$ Abstract
C
C     Read non-blank lines from a file into a buffer.
C
C$ Required_Reading
C
C     None.
C   
C$ Keywords
C
C     CELLS
C     CHARACTER
C     FILES
C
C$ Declarations
      
      INTEGER               UNIT
      INTEGER               START
      INTEGER               NLIN
      CHARACTER*(*)         BUFFER (*)
      LOGICAL               EOF
      
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     UNIT       I   Unit of file from which text is to be read.
C     START      I   Line in BUFFER where first line is to be placed.
C     NLIN      I/O  Number of lines to read into buffer.
C     BUFFER     O   Buffer to which text from file is to be placed.
C     EOF        O   True if end of file has been reached.
C      
C$ Detailed_Input
C      
C     UNIT       is the logical unit of a file, assumed to be 
C                formatted and opened sequentially.
C     
C     START      is the position in BUFFER where the first line read
C                from UNIT is to be placed.
C     
C     NLIN       is the number of lines to be read from UNIT.
C     
C$ Detailed_Output
C      
C     NLIN       is the number of lines read from UNIT. If EOF is 
C                false, NLIN will not be changed.
C     
C     BUFFER     is an array where the non-blank lines from UNIT are 
C                to be placed. BUFFER(START, START+NLIN-1) will be 
C                filled with lines from UNIT.
C                
C     EOF        is true if the end of file has been reached.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If an error is encountered while reading from the file
C        referenced by UNIT, the error SPICE(READFAILED) will be
C        signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     Lines longer than the size of an element in BUFFER will be 
C     truncated.
C     
C     This routine makes an attempt to read files that do not end with
C     a carriage-return character, followed by the end-of-file
C     character. For example, if [EOF] is the end-of-file indicator, 
C     FILBUF will read three lines from the file:
C     
C        When the beating of your heart echoes the 
C        beating of the drums, there is a life about 
C        to start when tomorrow comes.[EOF]
C     
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     B.V. Semenov   (JPL)
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 2.0.0, 03-AUG-1998 (BVS)
C
C        Routine was modified to ignore blank input lines. 
C
C-    Beta Version 1.0.0, 17-NOV-1992 (MJS) 
C
C-&

C
C     Local variables
C
      INTEGER               PNTER
      INTEGER               IOSTAT
      
      NLIN  = MAX (NLIN, 0)
      PNTER = 0
      EOF   = .FALSE.
      
      DO WHILE (.NOT. EOF .AND. PNTER .LT. NLIN)
         
         BUFFER (START+PNTER) = ' '
         
         READ (UNIT, FMT='(A)', IOSTAT=IOSTAT) BUFFER(START+PNTER)
         
         IF (IOSTAT .NE. 0) THEN
            
            IF (IOSTAT .GT. 0) THEN
               CALL CHKIN  ('FILBUF'                                )
               CALL SETMSG ('Error reading from file. UNIT = #, ' //
     .                      'IOSTAT = #.'                           )
               CALL ERRINT ('#', UNIT                               )
               CALL ERRINT ('#', IOSTAT                             )
               CALL SIGERR ('SPICE(FILEREADFAILED)'                 )
               CALL CHKOUT ('FILBUF'                                )
               RETURN
            END IF
            
            EOF = .TRUE.
            
            IF (BUFFER(START+PNTER) .EQ. ' ') THEN
               PNTER = PNTER - 1
            END IF
            
         END IF
         
C
C        We don't take blank lines, we don't need them ...
C
         IF ( BUFFER(START+PNTER) .NE. ' ' ) THEN
            PNTER = PNTER + 1
         END IF        
               
      END DO
      
      NLIN = PNTER
      
C     
C     That's it.
C     
      RETURN
      END
