C$Procedure      RDREC (Read a data record)

      SUBROUTINE RDREC (FILE, DATLIN, EOD)

C$ Abstract
C
C     Read a data line from an SFDU wrapped file.
C
C$ Required_Reading
C   
C      None.
C
C$ Keywords
C     
C     FILE
C     PARSING
C
C$ Declarations
      
      CHARACTER*(*)         FILE
      CHARACTER*(*)         DATLIN
      LOGICAL               EOD

C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FILE       I   Name of the wrapped file.
C     DATLIN     O   Data line.
C     EOD        O   True if end of data has been reached.
C      
C$ Detailed_Input
C      
C     FILE       is the name of an SFDU wrapped file.
C                
C$ Detailed_Output
C
C     LINE       is a line from the data portion of the SFDU wrapped
C                file.
C
C     EOD        is true if the last line of data has been read.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the SFDU can not be parsed, an appropriate error message
C        will result.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine understands both 'CCSD3ZS0' and 'CCSD3ZF0' type
C     SFDUs. The first type is referred to as type 1, and the second
C     type is referred to as type 2 in this routine.
C     
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     The file must contain at least one line of data.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 18-JUN-1999 (WLT)
C
C        Repaired errors in CHKIN/CHKOUT
C
C-    Beta Version 1.0.0, 13-NOV-1992 (MJS) 
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN
      LOGICAL               FAILED
      INTEGER               LTRIM
      INTEGER               RTRIM
      INTEGER               POSR

C
C     Local parameters
C
      INTEGER               CHUNK
      PARAMETER           ( CHUNK = 800 )
      
      INTEGER               FILEN
      PARAMETER           ( FILEN = 128 )
      
      INTEGER               TYPE1
      PARAMETER           ( TYPE1 = 1 )
      
      INTEGER               TYPE2
      PARAMETER           ( TYPE2 = 2 )
      
      INTEGER               LINLEN
      PARAMETER           ( LINLEN = 132 )
      
C
C     Local variables
C
      CHARACTER*(8)         SFDUID
      CHARACTER*(8)         DATAID
      CHARACTER*(8)         KID
      CHARACTER*(FILEN)     FILNAM
      CHARACTER*(LINLEN)    LINE
      
      INTEGER               SFDU
      INTEGER               DATA
      INTEGER               J
      INTEGER               LINPTR
      INTEGER               LINNUM
      INTEGER               BDATA
      INTEGER               EDATA
      
      LOGICAL               EOF
      LOGICAL               ERR

C
C     Saved variables
C
      SAVE

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDREC' )
      END IF

C
C     This routine should never be called. If this routine is called,
C     an error is signalled.
C
      CALL SETMSG ( 'RDREC: You have called an entry which  ' //
     .              'performs no run-time function. This may '        //
     .              'indicate a bug.'                                  )
 
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
 
      CALL CHKOUT ( 'RDREC' )
      RETURN



C$Procedure RDFREC (Read the first record from an SFDU file)

      ENTRY RDFREC (FILE, DATLIN)

C$ Abstract
C
C     Read the first data line from an SFDU wrapped file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PARSING
C     TEXT
C
C$ Declarations
C      
C     CHARACTER*(*)         FILE
C     CHARACTER*(*)         DATLIN
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FILE       I   Name of SFDU wrapped file.
C     LINE       O   Data line.
C
C$ Detailed_Input
C
C     FILE       is the name of an SFDU wrapped file.
C                
C$ Detailed_Output
C
C     DATLIN     is a line from the data portion of the SFDU wrapped
C                file.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the SFDU wrapped file does not contain at least one line of
C        data, the error SPICE(INVALIDSFDU) is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C   
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     The SCLK_SCET file must contain at least one record, as it is 
C     assumed to be an invalid file otherwise.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.1.0, 26-JAN-1993 (MJS) 
C
C        Checked FAILED after call to TXTRFL.
C
C-    Beta Version 1.0.0, 13-NOV-1992 (MJS) 
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('RDFREC')
      END IF
      
C     
C     Read the first line of the file.
C     
      FILNAM = FILE
      
      CALL TXTRFL (FILNAM, LINE)
      
      IF (FAILED()) THEN
         CALL CHKOUT ( 'RDFREC' )
         RETURN
      END IF
      
      DO WHILE (LINE .EQ. ' ')
         
         CALL TXTRRL (1, LINE, ERR, LINNUM)
         
         IF (ERR) THEN
            CALL SETMSG ('Unexpected end of file in #.')
            CALL ERRCH  ('#', FILNAM                   )
            CALL SIGERR ('SPICE(INVALIDSFDU)'          )
            CALL CHKOUT ('RDFREC'                      )
            RETURN
         END IF
         
      END DO
      
C     
C     We've got the first non-blank line. Determine the type of this 
C     SFDU.
C     
      CALL LJUST (LINE, LINE)
      
      IF (LINE(1:8) .EQ. 'CCSD3ZS0') THEN
         
         SFDU = TYPE1
         
C     
C        Remember the SFDU unique 8 character identification string.
C     
         SFDUID = LINE(13:20)
      
      ELSEIF (LINE(1:8) .EQ. 'CCSD3ZF0') THEN
         
         SFDU = TYPE2
      
      ELSE
         
         CALL SETMSG ('The type of SFDU in file # cannot be '  //
     .                'determined.'                             )
         CALL ERRCH  ('#', FILNAM                               )
         CALL SIGERR ('SPICE(INVALIDSFDU)'                      )
         CALL CHKOUT ('RDFREC'                                  )
         RETURN
         
      END IF
      
C     
C     The SFDU DDID (data description ID) should be '0001', regardless 
C     of the type.
C     
      IF (LINE(9:12) .NE. '0001') THEN
         CALL SETMSG ('The file # SFDU DDID is invalid: #.')
         CALL ERRCH  ('#', FILNAM                          )
         CALL ERRCH  ('#', LINE(9:12)                      )
         CALL SIGERR ('SPICE(INVALIDSFDU)'                 )
         CALL CHKOUT ('RDFREC'                             )
         RETURN
      END IF
      
      
C     
C     The K-header SFDU block should always be type 1 (where would the
C     data be if it was type 2?). The beginning K-header delimiter 
C     should directly follow the beginning SFDU delimiter, but we'll
C     allow it to appear on the following line.
C     
C     We don't know what the K-header DDID is. (It used to be 
C     L015, but it changes often these days...).
C     
      J = 21
      
      IF (LINE(J:) .EQ. ' ') THEN
      
         CALL TXTRRL (1, LINE, ERR, LINNUM)
         
         IF (ERR) THEN
            CALL SETMSG ('Unexpected end of file in #.')
            CALL ERRCH  ('#', FILNAM                   )
            CALL SIGERR ('SPICE(INVALIDSFDU)'          )
            CALL CHKOUT ('RDFREC'                      )
            RETURN
         END IF
      
         J = LTRIM (LINE)
         
      END IF
      
      
      IF (LINE(J:J+7) .NE. 'NJPL3KS0') THEN
         CALL SETMSG ('The begin K-header delimiter in file # is ' //
     .                'invalid: #.'                                )
         CALL ERRCH  ('#', FILNAM                                  )
         CALL ERRCH  ('#', LINE(J:J+11)                            )
         CALL SIGERR ('SPICE(INVALIDSFDU)'                         )
         CALL CHKOUT ('RDFREC'                                     )
         RETURN
      END IF

C     
C     Remember the K-header 8 character identification string.
C              
      J   = J+12
      KID = LINE(J:J+7)
      
C     
C     Search for the end K-header delimiter.
C     
      J = INDEX(LINE, 'CCSD3RE00000')
      
      DO WHILE (J .EQ. 0)
         
         CALL TXTRRL (1, LINE, ERR, LINNUM)
         
         IF (ERR) THEN
            CALL SETMSG ('Unexpected end of file in #.')
            CALL ERRCH  ('#', FILNAM                   )
            CALL SIGERR ('SPICE(INVALIDSFDU)'          )
            CALL CHKOUT ('RDFREC'                      )
            RETURN
         END IF
         
         J = INDEX(LINE, 'CCSD3RE00000')
         
      END DO
      
C     
C     Found it! The next 8 characters must equal KID.
C     
      J = J+12
      
      IF (LINE(J:J+7) .NE. KID) THEN
         CALL SETMSG ('Invalid end of K-header delimiter in file #.')
         CALL ERRCH  ('#', FILNAM                                   )
         CALL SIGERR ('SPICE(INVALIDSFDU)'                          )
         CALL CHKOUT ('RDFREC'                                      )
         RETURN
      END IF

C     
C     Next comes the I-block, or data section. Find it and determine its
C     type. 
C     
      J = J+8
      
      IF (LINE(J:) .EQ. ' ') THEN
      
         CALL TXTRRL (1, LINE, ERR, LINNUM)
         
         IF (ERR) THEN
            CALL SETMSG ('Unexpected end of file in #.')
            CALL ERRCH  ('#', FILNAM                   )
            CALL SIGERR ('SPICE(INVALIDSFDU)'          )
            CALL CHKOUT ('RDFREC'                      )
            RETURN
         END IF
      
         J = LTRIM (LINE)
         
      END IF
      
      IF (LINE(J:J+7) .EQ. 'NJPL3IS0') THEN
         
         DATA = TYPE1
C     
C        Remember the 8 character I-block identifier.
C     
         J      = J+12
         DATAID = LINE(J:J+7)
      
      ELSEIF (LINE(J:J+7) .EQ. 'NJPL3IF0') THEN
      
         J    = J+12
         DATA = TYPE2
         
      ELSE
      
         CALL SETMSG ('The begin data delimiter in file # is ' //
     .                'invalid: #.'                             )
         CALL ERRCH  ('#', FILNAM                               )
         CALL ERRCH  ('#', LINE(J:J+7)                          )
         CALL SIGERR ('SPICE(INVALIDSFDU)'                      )
         CALL CHKOUT ('RDFREC'                                  )
         RETURN
         
      END IF
      
C     
C     Now find the first data line. It may be appended to the begin
C     data delimiter.
C     
      J = J+8

      IF (LINE(J:) .EQ. ' ') THEN
      
         CALL TXTRRL (1, LINE, ERR, LINNUM)
         
         IF (ERR) THEN
            CALL SETMSG ('Unexpected end of file in #.')
            CALL ERRCH  ('#', FILNAM                   )
            CALL SIGERR ('SPICE(INVALIDSFDU)'          )
            CALL CHKOUT ('RDFREC'                      )
            RETURN
         END IF
      
         J = 1
      
      END IF

C     
C     BDATA is the position in LINE where the data starts.
C     
      BDATA  = J

C     
C     See if we can find the end of file.
C                 
      CALL TXTRRL (CHUNK, LINE, EOF, LINPTR)
      
      IF ( EOF .AND. (SFDU .EQ. TYPE1 .OR. DATA .EQ. TYPE1)) THEN
      
C      
C        Ending SFDU delimiter.
C     
         IF (SFDU .EQ. TYPE1) THEN
            
            DO WHILE (LINE .EQ. ' ' .AND. LINPTR .GE. LINNUM)
            
               CALL TXTRRL (-1, LINE, ERR, LINPTR)
               
            END DO
            
            IF (LINPTR .LT. LINNUM) THEN
               CALL SETMSG ('End SFDU delimiter not found in file #.')
               CALL ERRCH  ('#', FILNAM                              )
               CALL SIGERR ('SPICE(INVALIDSFDU)'                     )
               CALL CHKOUT ('RDFREC'                                 )
               RETURN
            END IF
            
            J = POSR  (LINE, 'CCSD3RE00000', RTRIM(LINE))
            
            IF (LINE(J+12:J+19) .NE. SFDUID) THEN
               CALL SETMSG ('Invalid end SFDU delimiter in file #.')
               CALL ERRCH  ('#', FILNAM                            )
               CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
               CALL CHKOUT ('RDFREC'                               )
               RETURN
            END IF
            
            LINE(J:) = ' '
            
         END IF
         
C     
C        Ending data delimiter.
C     
         IF (DATA .EQ. TYPE1) THEN
         
            DO WHILE (LINE .EQ. ' ' .AND. LINPTR .GE. LINNUM)
            
               CALL TXTRRL (-1, LINE, ERR, LINPTR)
               
            END DO
            
            IF (LINPTR .LT. LINNUM) THEN
               CALL SETMSG ('End data delimiter not found in file #.')
               CALL ERRCH  ('#', FILNAM                              )
               CALL SIGERR ('SPICE(INVALIDSFDU)'                     )
               CALL CHKOUT ('RDFREC'                                 )
               RETURN
            END IF
            
            J = POSR  (LINE, 'CCSD3RE00000', RTRIM(LINE))
            
            IF (LINE(J+12:J+19) .NE. DATAID) THEN
               CALL SETMSG ('Invalid end data delimiter in file #.')
               CALL ERRCH  ('#', FILNAM                            )
               CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
               CALL CHKOUT ('RDFREC'                               )
               RETURN
            END IF
            
            LINE(J:) = ' '
               
         END IF
         
         IF (J .EQ. 1) THEN
            CALL TXTRRL (-1, LINE, ERR, LINPTR)
            EDATA = RTRIM(LINE)
         ELSE
            EDATA = J - 1
         END IF
         
C     
C        Make sure we still have something.
C     
         IF (LINPTR .LT. LINNUM) THEN
            CALL SETMSG ('No data found in file #.')
            CALL ERRCH  ('#', FILNAM               )
            CALL SIGERR ('SPICE(INVALIDSFDU)'      )
            CALL CHKOUT ('RDFREC'                  )
            RETURN
         END IF
         
      ELSEIF (EOF) THEN
         
         EDATA = RTRIM (LINE)
      
      END IF
      
C     
C     
C     
      CALL TXTRRL (LINNUM-LINPTR, LINE, ERR, LINNUM)
      
      IF (LINPTR .NE. LINNUM) THEN
         DATLIN = LINE(BDATA:)
      ELSE
         DATLIN = LINE(BDATA:EDATA)
      END IF
      
      IF (.NOT. EOF) THEN
         LINPTR = LINNUM + (LINPTR-LINNUM)/2 
      END IF

      CALL CHKOUT ('RDFREC')
      RETURN



C$Procedure RDNREC (Read the next record from a SCLK_SCET file)

      ENTRY RDNREC (DATLIN, EOD)

C$ Abstract
C
C     Read the next data line from an SFDU wrapped file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PARSING
C     TEXT
C
C$ Declarations
C      
C     CHARACTER*(*)         DATLIN
C     LOGICAL               EOD
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     DATLIN     O   Data line.
C     EOD        O   True if the end of data has been reached.
C
C$ Detailed_Input
C
C     None.
C                
C$ Detailed_Output
C
C     DATLIN     is a line from the data portion of the SFDU wrapped
C                file.
C      
C     EOD        is true if the end of data has been reached.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the SFDU wrapped file does not contain at least one line of
C        data, the error SPICE(INVALIDSFDU) is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C   
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     The SCLK_SCET file must contain at least one record, as it is 
C     assumed to be an invalid file otherwise.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 13-NOV-1992 (MJS) 
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('RDNREC')
      END IF
      
C     
C     
C     
      IF (LINNUM+1 .LT. LINPTR) THEN
      
         CALL TXTRRL (1, LINE, ERR, LINNUM)
      
         DATLIN = LINE
         EOD    = .FALSE.
        
      ELSEIF (LINNUM+1 .GT. LINPTR) THEN
           
         DATLIN = ' '
         EOD    = .TRUE.
         
      ELSEIF (EOF) THEN
            
         CALL TXTRRL (1, LINE, ERR, LINNUM)
      
         DATLIN = LINE(1:EDATA)
         EOD    = .FALSE.
         
      ELSE
         
         CALL TXTRRL (CHUNK, LINE, EOF, LINPTR)
      
         IF ( EOF .AND. 
     .       (SFDU .EQ. TYPE1 .OR. DATA .EQ. TYPE1) ) THEN
      
C      
C           Ending SFDU delimiter.
C     
            IF (SFDU .EQ. TYPE1) THEN
            
               DO WHILE (LINE .EQ. ' ' .AND. LINPTR .GE. LINNUM)
            
                  CALL TXTRRL (-1, LINE, ERR, LINPTR)
               
               END DO
            
               IF (LINPTR .LT. LINNUM) THEN
                  CALL SETMSG ('End SFDU delimiter not found in '     //
     .                         'file #.'                              )
                  CALL ERRCH  ('#', FILNAM                            )
                  CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
                  CALL CHKOUT ('RDNREC'                               )
                  RETURN
               END IF
            
               J = POSR  (LINE, 'CCSD3RE00000', RTRIM(LINE))
            
               IF (LINE(J+12:J+19) .NE. SFDUID) THEN
                  CALL SETMSG ('Invalid end SFDU delimiter in file #.')
                  CALL ERRCH  ('#', FILNAM                            )
                  CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
                  CALL CHKOUT ('RDNREC'                               )
                  RETURN
               END IF
               
               LINE(J:) = ' '
               
            END IF
         
C     
C           Ending data delimiter.
C     
            IF (DATA .EQ. TYPE1) THEN
         
               DO WHILE (LINE .EQ. ' ' .AND. LINPTR .GE. LINNUM)
            
                  CALL TXTRRL (-1, LINE, ERR, LINPTR)
               
               END DO
            
               IF (LINPTR .LT. LINNUM) THEN
                  CALL SETMSG ('End data delimiter not found in '     //
     .                         'file #.'                              )
                  CALL ERRCH  ('#', FILNAM                            )
                  CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
                  CALL CHKOUT ('RDNREC'                               )
                  RETURN
               END IF
            
               J = POSR  (LINE, 'CCSD3RE00000', RTRIM(LINE))
            
               IF (LINE(J+12:J+19) .NE. DATAID) THEN
                  CALL SETMSG ('Invalid end data delimiter in file #.')
                  CALL ERRCH  ('#', FILNAM                            )
                  CALL SIGERR ('SPICE(INVALIDSFDU)'                   )
                  CALL CHKOUT ('RDNREC'                               )
                  RETURN
               END IF
            
               LINE(J:) = ' '
               
            END IF
         
            IF (J .EQ. 1) THEN
               CALL TXTRRL (-1, LINE, ERR, LINPTR)
               EDATA = RTRIM(LINE)
            ELSE
               EDATA = J - 1
            END IF
         
         ELSEIF (EOF) THEN
         
            EDATA = RTRIM (LINE)
         
         END IF

         CALL TXTRRL (LINNUM-LINPTR+1, LINE, ERR, LINNUM)
         
         DATLIN = LINE
         EOD    = .FALSE.
         
         IF (.NOT. EOF) THEN
            LINPTR = LINNUM + (LINPTR-LINNUM)/2 
         END IF
         
      END IF
         
      CALL CHKOUT ('RDNREC')
      RETURN
      END
