C$Procedure      RDSCET (Read SCLKSCET)

      SUBROUTINE RDSCET (FILE, SC, TICKS, UTCSEC, TDT, RATE, EOF)

C$ Abstract
C
C     Read a record from a SCLKSCET file.
C
C$ Required_Reading
C   
C     None.
C
C$ Keywords
C
C     PARSING
C     TEXT
C
C$ Declarations

      CHARACTER*(*)         FILE
      INTEGER               SC
      DOUBLE PRECISION      TICKS
      DOUBLE PRECISION      UTCSEC
      DOUBLE PRECISION      TDT
      DOUBLE PRECISION      RATE
      LOGICAL               EOF

C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FILE       I   Name of SCLKSCET file. 
C     SC         I   ID of spacecraft.
C     TICKS      O   Ticks.
C     UTCSEC     O   UTC seconds.
C     TDT        O   Terrestrial Dynamical Time.
C     RATE       O   Rate, the number of seconds per major clock count.
C     EOF        O   End of file.
C     
C$ Detailed_Input
C   
C     FILE       is the name of the SCLKSCET file.
C     
C     SC         is the NAIF ID of the spacecraft.
C     
C$ Detailed_Output
C      
C     TICKS,
C     TDT,
C     RATE        are components of a record. A record in a SCLKSCET 
C                 file contains a SCLK time, a SCET time, and a rate.
C                 The SCLK time is converted into ticks (TICKS), the
C                 SCET time is converted into TDT (TDT), and the rate,
C                 in units of seconds per major SC clock count, is 
C                 given as RATE.
C                 
C     UTCSEC      is the number of UTC seconds in TDT. This value is
C                 useful in computations, and thus is given in addition
C                 to TDT.
C     
C     EOF         is true if the end of file has been reached.
C
C$ Parameters
C
C     None.
C     
C$ Exceptions
C
C     1) If the SCLKSCET file is not formatted according to the format
C        given in reference [1], an appropriate error will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1] "SFOC-1-DPS-Any-SCLKvSCET", SFOC0038-00-25-01.
C
C$ Author_and_Institution
C
C     B.V. Semenov   (JPL)
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.2, 29-MAY-1999 (BVS) 
C
C        Modified RDFSCT to skip over multiple title ('*') lines.
C
C-    Beta Version 1.0.1, 04-SEP-1998 (BVS) 
C
C        ET time is now computed using a call to STR2ET rather than
C        a call to DELTET with subsequent addition of computed delta
C        to UTC seconds.
C
C-    Beta Version 1.0.0, 25-NOV-1992 (MJS) 
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      UNITIM

C
C     Local parameters
C
      INTEGER               LINLEN
      PARAMETER           ( LINLEN = 80 )
      
C
C     Local variables
C
      CHARACTER*(LINLEN)    LINE
      CHARACTER*(50)        SCET
      CHARACTER*(50)        UTC
      CHARACTER*(50)        SCLKCH
      CHARACTER*(50)        RATECH
      CHARACTER*(80)        ERROR
       
      DOUBLE PRECISION      ET
      
      INTEGER               LOC
      INTEGER               SCID
      
      LOGICAL               EOD
      
C
C     Saved variables
C
      SAVE


C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('RDSCET')
      END IF

C
C     This routine should never be called. If this routine is called,
C     an error is signalled.
C
      CALL SETMSG ( 'RDREC: You have called an entry which  ' //
     .              'performs no run-time function. This may '        //
     .              'indicate a bug.'                                  )
 
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
 
      CALL CHKOUT ( 'RDSCET' )
      RETURN




C$Procedure RDFSCT (Read SCET, first record)

      ENTRY RDFSCT (FILE, SC, TICKS, UTCSEC, TDT, RATE)

C$ Abstract
C
C     Read the first record from a SCLKSCET file.
C      
C$ Required_Reading
C
C     None.
C
C$ Keywords
C      
C     PARSING
C     TEXT
C
C$ Declarations
C
C     CHARACTER*(*)         FILE
C     DOUBLE PRECISION      TICKS
C     DOUBLE PRECISION      UTCSEC
C     DOUBLE PRECISION      TDT
C     DOUBLE PRECISION      RATE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FILE       I   Name of SCLKSCET file. 
C     SC         I   ID of spacecraft.
C     TICKS      O   Ticks.
C     UTCSEC     O   UTC seconds.
C     TDT        O   Terrestrial Dynamical Time.
C     RATE       O   Rate, the number of seconds per major clock count.
C
C$ Detailed_Input
C
C     FILE       is the name of the SCLKSCET file.
C     
C     SC         is the NAIF ID of the spacecraft.
C     
C$ Detailed_Output
C      
C     TICKS,
C     TDT,
C     RATE        are components of a record. A record in a SCLKSCET 
C                 file contains a SCLK time, a SCET time, and a rate.
C                 The SCLK time is converted into ticks (TICKS), the
C                 SCET time is converted into TDT (TDT), and the rate,
C                 in units of seconds per major SC clock count, is 
C                 given as RATE.
C                 
C     UTCSEC      is the number of UTC seconds in TDT. This value is
C                 useful in computations, and thus is given in addition
C                 to TDT.
C     
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     See above.
C     
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.2, 29-MAY-1999 (BVS) 
C
C        Modified this enstry point to skip over multiple title 
C        ('*') lines.
C
C-    Beta Version 1.0.0, 25-NOV-1992 (MJS) 
C
C-&


C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('RDFSCT')
      END IF

C
C     Read the first line. Should be a title line that starts with
C     a '*'.
C
      CALL RDFREC (FILE, LINE)
      CALL LJUST  (LINE, LINE)

      IF (LINE(1:1) .NE. '*') THEN
         CALL SETMSG ('Invalid SCLK_SCET file.')
         CALL SIGERR ('SPICE(INVALIDSCLKSCET)' )
         CALL CHKOUT ('RDFSCT'                 )
         RETURN
      END IF
      
C
C     Sometimes SCET files contain more than one title record. We 
C     need to check if our file does and skip all title records.
C
      CALL RDNREC (LINE, EOD)
      CALL LJUST  (LINE, LINE)
      
      DO WHILE( .NOT. EOD .AND. LINE(1:1) .EQ. '*' )
      
         CALL RDNREC (LINE, EOD)
         CALL LJUST  (LINE, LINE)
      
      END DO
      
C     
C     We skipped all title lines and check whether current non-title
C     line is a data line. We should have at least one line 
C     of data.
C     
      IF (EOD) THEN
         CALL SETMSG ('SCLK_SCET file contains no data.')
         CALL SIGERR ('SPICE(INVALIDSCLKSCET)'          )
         CALL CHKOUT ('RDFSCT'                          )
         RETURN
      END IF
      
C     
C     The first field is the SCLK time. Convert it in to ticks.
C     
      SCID = SC
      
      CALL NTHWD  (LINE, 1, SCLKCH, LOC)
      CALL SCTIKS (SCID, SCLKCH, TICKS)
      
C     
C     The next field is the SCET time. Get the number of UTC seconds, 
C     and the TDT time.
C     
      CALL NTHWD (LINE, 2, SCET, LOC)
      
      LOC = INDEX (SCET, 'T')
      
      IF (LOC .EQ. 0) THEN
         CALL SETMSG ('Unable to parse SCET time #.')
         CALL ERRCH  ('#', UTC                      )
         CALL SIGERR ('SPICE(INVALIDTIME)'          )
         CALL CHKOUT ('RDFSCT'                      )
         RETURN
      END IF
      
      UTC = SCET(:LOC-1) // '//' // SCET(LOC+1:)
      
      CALL TPARSE (UTC, UTCSEC, ERROR)
      
      IF (ERROR .NE. ' ') THEN
         CALL SETMSG ('Invalid SCET time: #. #.')
         CALL ERRCH  ('#', SCET                 )
         CALL ERRCH  ('#', ERROR                )
         CALL SIGERR ('SPICE(INVALIDTIME)'      )
         CALL CHKOUT ('RDFSCT'                  )
         RETURN
      END IF

      CALL STR2ET( UTC, ET )
      TDT = UNITIM (ET, 'ET', 'TDT')
            
C     
C     The rate is in the fourth field.
C     
      CALL NTHWD  (LINE,   4,    RATECH, LOC)
      CALL NPARSD (RATECH, RATE, ERROR,  LOC)
      
      IF (ERROR .NE. ' ') THEN
         CALL SETMSG ('Invalid rate: #. #.')
         CALL ERRCH  ('#', RATECH          )
         CALL ERRCH  ('#', ERROR           )
         CALL SIGERR ('SPICE(INVALIDRATE)' )
         CALL CHKOUT ('RDFSCT'             )
         RETURN
      END IF
      
C     
C     That's it.
C     
      CALL CHKOUT ('RDFSCT')
      RETURN




C$Procedure RDNSCT (Read SCET, next record)

      ENTRY RDNSCT (TICKS, UTCSEC, TDT, RATE, EOF)

C$ Abstract
C
C     Read the next data record from a SCLKSCET file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C      
C     PARSING
C     TEXT
C
C$ Declarations
C
C     DOUBLE PRECISION      TICKS
C     DOUBLE PRECISION      UTCSEC
C     DOUBLE PRECISION      TDT
C     DOUBLE PRECISION      RATE
C     LOGICAL               EOF
C   
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C
C$ Detailed_Input
C      
C     None.
C
C$ Detailed_Output
C
C     TICKS,
C     TDT,
C     RATE        are components of a record. A record in a SCLKSCET 
C                 file contains a SCLK time, a SCET time, and a rate.
C                 The SCLK time is converted into ticks (TICKS), the
C                 SCET time is converted into TDT (TDT), and the rate,
C                 in units of seconds per major SC clock count, is 
C                 given as RATE.
C                 
C     UTCSEC      is the number of UTC seconds in TDT. This value is
C                 useful in computations, and thus is given in addition
C                 to TDT.
C      
C     EOF         is true if the last record has been read.
C
C$ Parameters
C
C     None.
C   
C$ Exceptions
C
C     See above.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 25-NOV-1992 (MJS) 
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('RDNSCT')
      END IF

C     
C     Read the next line.
C     
      CALL RDNREC (LINE, EOD)
      
      IF (EOD) THEN
         EOF = .TRUE.
         CALL CHKOUT ('RDNSCT')
         RETURN
      END IF
      
      EOF = .FALSE.
      
C     
C     The first field is the SCLK time. Convert it in to ticks.
C     
      CALL NTHWD  (LINE, 1, SCLKCH, LOC)
      CALL SCTIKS (SCID, SCLKCH, TICKS)
      
C     
C     The next field is the SCET time. Get the number of UTC seconds, 
C     and the TDT time.
C     
      CALL NTHWD (LINE, 2, SCET, LOC)
      
      LOC = INDEX (SCET, 'T')
      
      IF (LOC .EQ. 0) THEN
         CALL SETMSG ('Unable to parse SCET time #.')
         CALL ERRCH  ('#', UTC                      )
         CALL SIGERR ('SPICE(INVALIDTIME)'          )
         CALL CHKOUT ('RDNSCT'                      )
         RETURN
      END IF
      
      UTC = SCET(:LOC-1) // '//' // SCET(LOC+1:)
      
      CALL TPARSE (UTC, UTCSEC, ERROR)
      
      IF (ERROR .NE. ' ') THEN
         CALL SETMSG ('Invalid SCET time: #. #.')
         CALL ERRCH  ('#', SCET                 )
         CALL ERRCH  ('#', ERROR                )
         CALL SIGERR ('SPICE(INVALIDTIME)'      )
         CALL CHKOUT ('RDNSCT'                  )
         RETURN
      END IF
      
      CALL STR2ET( UTC, ET )
      TDT = UNITIM (ET, 'ET', 'TDT')
      
C     
C     The rate is in the fourth field.
C     
      CALL NTHWD  (LINE,   4,    RATECH, LOC)
      CALL NPARSD (RATECH, RATE, ERROR,  LOC)
      
      IF (ERROR .NE. ' ') THEN
         CALL SETMSG ('Invalid rate: #. #.')
         CALL ERRCH  ('#', RATECH          )
         CALL ERRCH  ('#', ERROR           )
         CALL SIGERR ('SPICE(INVALIDRATE)' )
         CALL CHKOUT ('RDNSCT'             )
         RETURN
      END IF
      
C     
C     That's it.
C     
      CALL CHKOUT ('RDNSCT')
      RETURN
      END

