#!/usr/bin/perl
#
# This script illustrates how to run TCP2SCET and MAKCLK to convert MEX
# TCPs into SPICE SCLK.
#
# Usage:
#
#   > make_mex_sclk.pl <bad_times_file>        \
#                      <old_complete_tcp_file> \
#                      <new_tcp_file>          \
#                      <new_complete_tcp_file> \
#                      <sclk_file>
#
# where:
#
#   bad_times_file         input file containing times of bad TCPs, one
#                          time per line, quoted by single quotes
#
#   old_complete_tcp_file  input file containing all TCPs from the beginning
#                          of the mission
#
#   new_tcp_file           input file containing new TCP packet(s)
#
#   new_complete_tcp_file  output file containing all TCPs from the beginning
#                          of the mission and new TCP
#
#   sclk_file              output SCLK kernel
#
# Usage example:
#
#   > make_mex_sclk.pl bad_times_to_2003-07-18.txt          \
#                      tcps_up_to_2003-07-18.bin            \
#                      MEX_TLM_TCP_2003-08-21T11-34-48.BIN  \
#                      tcps_up_to_2003-08-21.bin            \
#                      mex_030915_step.tsc
#
# The script prints all diagnostic/status messages to STDOUT.
#
# BVS/NAIF, October 13, 2003
#
$scriptname = "make_mex_sclk.pl";
$version    = "1.0.0 -- October 13, 2003 -- BVS/NAIF";

#
# Re-direct STDERR to STDOUT to capture all output in one stream.
#
open ( STDERR, ">&STDOUT" );

#
# Use File::stat to check that TCP file sizes are multiples of 48.
#
use File::stat;

#
# Check command line arguments.
#
if ( @ARGV != 5 ){

   #
   # We don't have exactly 5 arguments. Display usage and stop.
   #
   die <<EndofUsage;
ERROR:$scriptname: Incorrect number of command line arguments.

$scriptname usage is:

   \% $scriptname <bad_times_file>        \\
                      <old_complete_tcp_file> \\
                      <new_tcp_file>          \\
                      <new_complete_tcp_file> \\
                      <sclk_file>

See the script header for more details.

EndofUsage

} else {

   #
   # Check input/output files. Inputs must exist and be readable. Outputs
   # should not exist.
   #
   if ( ! ( -r $ARGV[0] ) ) {
      die "\nERROR:$scriptname: '$ARGV[0]' doesn't exist or not readable.\n\n";
   } else {
      $badtimes = $ARGV[0];
   }
   if ( ! ( -r $ARGV[1] ) ) {
      die "\nERROR:$scriptname: '$ARGV[1]' doesn't exist or not readable.\n\n";
   } else {
      $oldctcp = $ARGV[1];
      $stats = stat( $oldctcp );
      if ( $stats->size % 48 ) {
         die "\nERROR:$scriptname: the size of '$oldctcp' is not divisible ".
             "by 48 (single TCP size). Is it really a TCP file?\n\n";
      }
   }
   if ( ! ( -r $ARGV[2] ) ) {
      die "\nERROR:$scriptname: '$ARGV[2]' doesn't exist or not readable.\n\n";
   } else {
      $newitcp = $ARGV[2];
      $stats = stat( $newitcp );
      if ( $stats->size % 48 ) {
         die "\nERROR:$scriptname: the size of '$newitcp' is not divisible ".
             "by 48 (single TCP size). Is it really a TCP file?\n\n";
      }
   }
   if ( -e $ARGV[3] ) {
      die "\nERROR:$scriptname: '$ARGV[3]' already exists.\n\n";
   } else {
      $newctcp = $ARGV[3];
   }
   if ( -e $ARGV[4] ) {
      die "\nERROR:$scriptname: '$ARGV[4]' already exists.\n\n";
   } else {
      $sclk = $ARGV[4];
   }
}

#
# External executables and files that we need:
#
#  $tcp2scet -- TCP2SCET executable
#  $makclk   -- MAKCLK executable
#  $lsk      -- latest LSK file name
#
$lsk      = "./naif0007.tls";
$tcp2scet = "./tcp2scet/tcp2scet";
$makclk   = "./makclk/makclk";

#
# Check is these exist and OK.
#
if ( ! ( -r $lsk ) ) {
   die "\nERROR:$scriptname: '$lsk' doesn't exist or not readable.\n\n";
}
if ( ! ( -x $tcp2scet ) ) {
   die "\nERROR:$scriptname: '$tcp2scet' doesn't exist or not executable.\n\n";
}
if ( ! ( -x $makclk ) ) {
   die "\nERROR:$scriptname: '$makclk' doesn't exist or not executable.\n\n";
}

#
# Some few intermediate files that will be made and a flag saying if they
# should be deleted after the run (should be set to "yes" or "no".)
#
$tcp2scetsetup = $sclk.".tcp2scet";
$scet          = $sclk.".scet";
$makclksetup   = $sclk.".makclk";
$makclklog     = $sclk.".makclklog";
$oldsclk       = $sclk.".oldsclk";
$keeptmps      = "yes";

#
# Check that they don't exist.
#
if ( -e $tcp2scetsetup ) {
   die "\nERROR:$scriptname: '$tcp2scetsetup' already exists.\n\n";
}
if ( -e $scet ) {
   die "\nERROR:$scriptname: '$scet' already exists.\n\n";
}
if ( -e $makclksetup ) {
   die "\nERROR:$scriptname: '$makclksetup' already exists.\n\n";
}
if ( -e $makclklog ) {
   die "\nERROR:$scriptname: '$makclklog' already exists.\n\n";
}
if ( -e $oldsclk ) {
   die "\nERROR:$scriptname: '$oldsclk' already exists.\n\n";
}

#
# Copy old complete TCP to the new complete TCP.
#
$result = `cp $oldctcp $newctcp 2>&1`;
$status = ( $? >> 8 );
if ( $status ) {
   die "\nERROR:$scriptname: failed to copy $oldctcp to $newctcp. ".
       "Error was:\n\n$result\n";
}

#
# Append new individual TCP to the new complete TCP using UNIX cat command.
#
$result = `cat $newitcp >> $newctcp`;
$status = ( $? >> 8 );
if ( $status ) {
   die "\nERROR:$scriptname: failed to append $newitcp to $newctcp. ".
       "Error was:\n\n$result\n";
}

#
# Set TCP2SCET setup parameters -- clock start, bad packet times,
# discontinuity threshold, and auxiliary interval size.
#
$tcp_start    = "2003-153T17:00";
$tcp_badtimes = `cat $badtimes`;
$tcp_discont  = "0.2";
$tcp_interval = "60.0";
$tcp_mode     = "step";

#
# Make TCP2SCET setup file.
#
open ( TCPSETUP, ">$tcp2scetsetup" )
   || die "ERROR:$scriptname: cannot open '$tcp2scetsetup' for write.\n\n";

print TCPSETUP <<EndOfTCPSETUP;

TCP2SCET setup file.

\\begindata
   LEAPSECONDS_FILE        = '$lsk'
   CLOCK_START_TIME        = '2003-153T17:00'
   BAD_PACKET_TIMES        = (
$tcp_badtimes                             )

   DISCONTINUITY_THRESHOLD = $tcp_discont
   AUXILIARY_INTERVAL_SIZE = $tcp_interval
   MISSION_NAME            = 'MARS_EXPRESS'
   MISSION_ID              = 41
   SPACECRAFT_NAME         = 'MARS_EXPRESS'
   SPACECRAFT_ID           = 41
   PRODUCER_ID             = 'PST/ESTEC'
\\begintext

EndOfTCPSETUP

close ( TCPSETUP )
   || die "ERROR:$scriptname: cannot close '$tcp2scetsetup'\n\n";

#
# Run TCP2SCET.
#
$result = `$tcp2scet -setup $tcp2scetsetup -mode $tcp_mode -tcp $newctcp -scet $scet -silent 2>&1`;
$status = ( $? >> 8 );
if ( $status ) {
   die "\nERROR:$scriptname: TCP2SCET failed. Error was:\n\n$result\n";
}

#
# Make SCLK template needed for MAKCLK.
#
open ( OLDSCLK, ">$oldsclk" )
  || die "ERROR:$scriptname: cannot open '$oldsclk' for write.\n\n";

print OLDSCLK <<EndOfOLDSCLK;

MEX SCLK Template for initial MAKCLK run. BVS/NAIF, 04/23/03.

Assumed clock granularity is 1/65536 of a second.

\\begindata
SCLK_KERNEL_ID           = ( \@2003-04-23/15:00 )
SCLK_DATA_TYPE_41        = ( 1 )
SCLK01_TIME_SYSTEM_41    = ( 2 )
SCLK01_N_FIELDS_41       = ( 2 )
SCLK01_MODULI_41         = ( 4294967296 65536 )
SCLK01_OFFSETS_41        = ( 0 0 )
SCLK01_OUTPUT_DELIM_41   = ( 1 )
SCLK_PARTITION_START_41  = ( 0.0000000000000E+00 )
SCLK_PARTITION_END_41    = ( 2.8147497671065E+14 )
SCLK01_COEFFICIENTS_41   = (
    0.0000000000000E+00     0.0000000000000E+08     1.0000000000000E+00
                           )
\\begintext

EndOfOLDSCLK

close ( OLDSCLK ) || die "ERROR:$scriptname: cannot close '$oldsclk'\n\n";

#
# Make MAKCLK setup file.
#
open ( MAKSETUP, ">$makclksetup" )
  || die "ERROR:$scriptname: cannot open '$makclksetup' for write.\n\n";

print MAKSETUP <<EndOfMAKSETUP;
SCLKSCET_FILE          = $scet
OLD_SCLK_KERNEL        = $oldsclk
FILE_NAME              = $sclk
NAIF_SPACECRAFT_ID     = -41
LEAPSECONDS_FILE       = $lsk
PARTITION_TOLERANCE    = 6554
LOG_FILE               = $makclklog
EndOfMAKSETUP

close ( MAKSETUP ) || die "ERROR:$scriptname: cannot close '$makclksetup'\n\n";

#
# Run MAKCLK to convert SCET to SCLK. Setup file name needs to be provided
# through STDIN.
#
$command = "$makclk 2>&1 <<EndTransfer\n".
           "$makclksetup\n".
           "EndTransfer\n";
$result = `$command`;
$status = ( $? >> 8 );
if ( $status ) {
   die "\nERROR:$scriptname: MAKCLK failed. Error was:\n\n$result\n";
}

#
# Conversion went OK and the file was created but it has no comments.
# Some comments must be added. Extract SCLK data, SCET SFDU header and
# setup contents into internal variables.
#
$result = `cat $sclk`;
( $result, $sclkdata ) = split( /\\begindata\n/, $result );

$result = `cat $scet`;
( $sfdu, $result ) = split( /\*_+SCLK0_+/, $result );
$sfdu =~ s/\n/\n     /g;
$sfdu = "     ".$sfdu;
$sfdu =~ s/_/-/g;
$sfdu =~ s/\s*CCSD.*\$\$\s*\n//g;

$setups = `cat $makclksetup`;
$setups =~ s/\n/\n     /g;
$setups = "     ".$setups;

#
# Re-open the same SCLK file for write, print extracted SCLK data with the
# appropriate comments to it and close the file.
#
open ( SCLK, ">$sclk" )
  || die "ERROR:$scriptname: Cannot re-open '$sclkfile' for write.\n\n";

print SCLK <<EndOfPassage;
KPL/SCLK


MEX SCLK File
===========================================================================

     This file is a SPICE spacecraft clock (SCLK) kernel containing
     information required for MEX spacecraft on-board clock to UTC
     conversion.


Production/History of this SCLK file
--------------------------------------------------------

     This file was generated by the NAIF utility program MAKCLK
     from the most recent MEX Time Correlation Packet (TCP) data
     converted to SCLKvSCET format using TCP2SCET program.


Usage
--------------------------------------------------------

     This file must be loaded into the users program by a call to the
     FURNSH subroutine

          CALL FURNSH( 'this_file_name' )

     in order to use the SPICELIB SCLK family of subroutines to convert
     MEX spacecraft on-board clock to ET and vice versa.


SCLK Format
--------------------------------------------------------

     The on-board clock, the conversion for which is provided by this SCLK
     file, consists of two fields:

          SSSSSSSSSS:FFF

     where:

          SSSSSSSSSS -- count of on-board seconds

          FFF        -- count of fractions of a second with one fraction
                        being 1/65536 of a second


References
--------------------------------------------------------

         1.   SCLK Required Reading Document

         2.   TCP2SCET Users Guide Document

         3.   MAKCLK Users Guide Document


Inquiries
--------------------------------------------------------

     If you have any questions regarding this file contact PST at ESTEC

           Jorge Diaz del Rio
           jdiaz\@rssd.esa.int

           Joe Zender
           Joe.Zender\@esa.int

     or NAIF at JPL

           Charles H. Acton, Jr
           (818) 354-3869
           Chuck.Acton\@jpl.nasa.gov

           Boris V. Semenov
           (818) 354-8136
           Boris.Semenov\@jpl.nasa.gov


SCLKvSCET File Header
--------------------------------------------------------

$sfdu


MAKCLK Setup file
--------------------------------------------------------

$setups


Kernel DATA
--------------------------------------------------------

\\begindata

$sclkdata

EndOfPassage

close ( SCLK );

#
# Do cleanup
#
if ( $keeptmps eq "no" ) {
  unlink ( $tcp2scetsetup ) ||
     die "\nERROR:$scriptname: cannot remove '$tcp2scetsetup'.\n\n";
  unlink ( $scet ) ||
     die "\nERROR:$scriptname: cannot remove '$scet'.\n\n";
  unlink ( $makclksetup ) ||
     die "\nERROR:$scriptname: cannot remove '$makclksetup'.\n\n";
  unlink ( $makclklog ) ||
     die "\nERROR:$scriptname: cannot remove '$makclklog'.\n\n";
  unlink ( $oldsclk ) ||
     die "\nERROR:$scriptname: cannot remove '$oldsclk'.\n\n";
}

#
# All done.
#
exit;
