C$ Procedure     TCPCHK -- Check and Output TCP Discontinuities

      SUBROUTINE TCPCHK ( MSG, ON, NREC, DDHTIM, DDHS, DDHMS, 
     .                    TCPG, TCPO, TCPTS, UTC, ET, MAXDSC, OK ) 

C$ Abstract
C
C     This routine computes discontinuities between TCP records, checks
C     them against specified threshold, and, if requested, outputs them
C     along with all TCP data to STDOUT as a nice formatted table.
C
C     This routine calls SPEAK routine to do output. If SPEAK[er] is 
C     OFF, it will print nothing.
C
C$ Copyright
C
C     Copyright (2003), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT  NONE

      INCLUDE               'tcp2scet.inc'

      CHARACTER*(*)         MSG
      LOGICAL               ON
      INTEGER               NREC
      DOUBLE PRECISION      DDHTIM ( * )
      INTEGER               DDHS   ( * )
      INTEGER               DDHMS  ( * )
      DOUBLE PRECISION      TCPO   ( * )
      DOUBLE PRECISION      TCPG   ( * )
      INTEGER               TCPTS  ( * )
      CHARACTER*(*)         UTC    ( * )
      DOUBLE PRECISION      ET     ( * )
      DOUBLE PRECISION      MAXDSC
      LOGICAL               OK

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     MSG        I   Message to be printed before table
C     ON         I   Flag indicating to output or not (.TRUE. to print)
C     NREC       I   Number of TCP records in the input buffers
C     DDHTIM     I   DDH time, as DP seconds
C     DDHS       I   DDH time, integer seconds 
C     DDHMS      I   DDH time, fractional seconds
C     TCPO       I   TCP offset
C     TCPG       I   TCP gradient
C     TCPTS      I   TCP generation time
C     UTC        I   UTC corresponding to DDH time
C     ET         I   ET corresponding to DDH time
C     MAXDSC     I   TCP discontinuity threshold
C     OK         O   Flag indicating that all discontinuity are with 
C                    threshold (if yes, set to .TRUE., otherwise .FALSE.)
C
C$ Detailed_Input
C
C     N/A.
C
C$ Detailed_Output
C
C     N/A
C
C$ Parameters
C
C     See 'tcp2scet.inc'.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     N/A.
C
C$ Examples
C
C     N/A.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     B.V. Semenov     (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    TCP2SCET Version 1.0.0, 17-SEP-2003 (BVS)
C
C-&
 
C
C     Local variables
C
      CHARACTER*(LINSIZ)    LINE
      CHARACTER*(WRDSIZ)    HWORD
      CHARACTER*(WRDSIZ)    TAG

      DOUBLE PRECISION      OBTCUR
      DOUBLE PRECISION      OBTDEL
      DOUBLE PRECISION      OBTPRV

      INTEGER               RECIDX
      INTEGER               RTRIM
      
C
C     If print is ON, print the header.
C
      IF ( ON ) THEN
         CALL SPEAK( ' ' )
         CALL SPEAK( MSG )
         CALL SPEAK( ' ' )
         CALL SPEAK( 'UTC                   DHH seconds    ' //
     .               'Offset         Gradient       OBT(i)       ' //
     .               'OBT(i-1)     Delta     Good?' )
         CALL SPEAK( '--------------------- -------------- ' //
     .               '-------------- -------------- ------------ ' //
     .               '------------ --------- -------' )
      END IF

C
C     Set output logical OK to .TRUE.
C
      OK = .TRUE.

C
C     Loop over TCP records.
C
      DO RECIDX = 1, NREC

C
C        Compute discontinuities.
C
         IF ( RECIDX .NE. 1 ) THEN
            OBTCUR = ( DDHTIM(RECIDX) - TCPO(RECIDX) )/ TCPG(RECIDX)
            OBTPRV = ( DDHTIM(RECIDX) - TCPO(RECIDX-1) )/ TCPG(RECIDX-1)
            OBTDEL = OBTCUR - OBTPRV
         ELSE
            OBTCUR = ( DDHTIM(RECIDX) - TCPO(RECIDX) )/ TCPG(RECIDX)
            OBTPRV = OBTCUR
            OBTDEL = 0.D0
         END IF

C
C        Check how this discontinuity compares to the thresholds. If
C        it's too big, reset OK to .FALSE.
C
         IF ( DABS(OBTDEL) .GT. MAXDSC ) THEN
            TAG = 'TOO BIG'
            OK = .FALSE.
         ELSE
            TAG = 'ok' 
         END IF

C
C        If print is ON, assemble a nice output line and print it.
C
         IF ( ON ) THEN

            LINE = UTC(RECIDX)(1:22)

            CALL DPFMT( DDHTIM(RECIDX), 'xxxxxxxxxxx.xxx', HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            CALL DPFMT( TCPO(RECIDX),   'xxxxxxxxxxx.xxx', HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            CALL DPFMT( TCPG(RECIDX),   'xx.xxxxxxxxxxxx', HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            CALL DPFMT( OBTCUR,         'xxxxxxxxx.xxx',   HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            CALL DPFMT( OBTPRV,         'xxxxxxxxx.xxx',   HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            CALL DPFMT( OBTDEL,         'xxxxxx.xxx',      HWORD )
            LINE = LINE(:RTRIM(LINE)) // HWORD

            LINE = LINE(:RTRIM(LINE)) // ' ' // TAG(:RTRIM(TAG))

            CALL SPEAK( LINE )

         END IF

      END DO

      IF ( ON ) THEN
         CALL SPEAK( ' ' )
      END IF

      RETURN
      END
